/*
  Copyright (c) 2013 by Switch Science (sales at switch-science.com)
  Cypress(Ramtron) FM25W256 SPI F-RAM library for Arduino

  Original library is
    Copyright (c) 2010 by arms22 (arms22 at gmail.com)
    Microchip 23x256 SPI SRAM library for Arduino

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
*/

#include "SPIFRAM.h"

// INSTRUCTION SET
#define READ	0x03			// Read data from memory
#define WRITE	0x02			// Write data to memory
#define RDSR	0x05			// Read Status register
#define WRSR	0x01			// Write Status register
#define WREN	0x06			// Write Enable


SPIFRAM::SPIFRAM(byte ncsPin)
	: _ncsPin(ncsPin)
{
}

byte SPIFRAM::read(unsigned int address)
{
	_buf[0] = READ;
	_buf[1] = address >> 8;
	_buf[2] = address & 0xff;
	_buf[3] = 0x00;
	return transfer(4);
}

void SPIFRAM::read(unsigned int address, byte *buffer, unsigned int size)
{
	select();
	SPI.transfer(READ);
	SPI.transfer(address >> 8);
	SPI.transfer(address & 0xff);
	for(unsigned int i=0; i<size; i++){
		*buffer++ = SPI.transfer(0);
	}
	deselect();
}

void SPIFRAM::write(unsigned int address, byte data)
{
	_buf[0] = WREN;
	transfer(1);
	_buf[0] = WRITE;
	_buf[1] = address >> 8;
	_buf[2] = address & 0xff;
	_buf[3] = data;
	transfer(4);
}

void SPIFRAM::write(unsigned int address, byte *buffer, unsigned int size)
{
	_buf[0] = WREN;
	transfer(1);
	select();
	SPI.transfer(WRITE);
	SPI.transfer(address >> 8);
	SPI.transfer(address & 0xff);
	for(unsigned int i=0; i<size; i++){
		SPI.transfer(*buffer++);
	}
	deselect();
}

void SPIFRAM::select(void)
{
	digitalWrite(_ncsPin, LOW);
}

void SPIFRAM::deselect(void)
{
	digitalWrite(_ncsPin, HIGH);
}

byte SPIFRAM::transfer(byte length)
{
	byte ret,i;
	select();
	for(i=0; i<length; i++){
		ret = SPI.transfer(_buf[i]);
	}
	deselect();
	return ret;
}
